@@ -27,13 +27,25 @@
 # 2017-08-23: plot.eff, in key.args, set default for between.columns=0
 # 2017-08-20: reintroduce legacy arguments for plot.eff()
 # 2017-09-10: use replacement for grid.panel()
+# 2017-11-03: Added a test to assume that at least one point will be plotted in a tile, else
+#                draw a blank tile. Needed for rank-deficient models.  S. Weisberg.
+# 2018-01-02: Changed the default key:  see lines 240-241
+# 2018-01-02: Rewrote find.legend columns, lines 41-44
+# 2018-01-30: enlarged text in key titles
 
 # the following functions aren't exported
 
-find.legend.columns <- function(n, target=min(4, n)){
-  rem <- n %% target
-  if (rem != 0 && rem < target/2) target <- target - 1
-  target
+#find.legend.columns <- function(n, target=min(4, n)){
+#  rem <- n %% target
+#  if (rem != 0 && rem < target/2) target <- target - 1
+#  target
+#}
+# new version 1/2/2017 by sw
+find.legend.columns <- function(n, space="top"){
+  if(space == "right") 1 else {
+  if(n <= 2) 2 else {
+   if(n == 3) 1 else 
+     {if (n <= 6) 2 else 3}}}
 }
 
 
@@ -47,7 +59,7 @@ make.ticks <- function(range, link, inverse, at, n) {
   if (is.null(n)) n <- 5
   labels <- if (is.null(at)){
     range.labels <- sapply(range, inverse)
-    labels <- grid.pretty(range.labels)
+    labels <- grid::grid.pretty(range.labels)
   }
   else at
   ticks <- try(sapply(labels, link), silent=TRUE)
@@ -92,16 +104,18 @@ panel.bands <- function(x, y, upper, lower, fill, col,
 
 spline.llines <- function(x, y, ...) llines(spline(x, y), ...)
 
-plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effect plot"),
-                     symbols=TRUE, lines=TRUE, axes, confint, partial.residuals, id, lattice,
-                     ...,
-                     # legacy arguments:
-                     multiline, rug, xlab, ylab, colors, cex, lty, lwd, ylim, xlim, factor.names, ci.style,
-                     band.transparency, band.colors, type, ticks, alternating, rotx, roty, grid, layout,
-                     rescale.axis, transform.x, ticks.x, show.strip.values, key.args, use.splines,
-                     residuals.color, residuals.pch, residuals.cex, smooth.residuals,
-                     residuals.smooth.color, show.fitted, span)
-{
+plot.eff <- function(x, x.var, z.var=which.min(levels), 
+          main=paste(effect, "effect plot"),
+          symbols=TRUE, lines=TRUE, axes, confint, partial.residuals, id, lattice,
+          ...,
+        # legacy arguments:
+          multiline, rug, xlab, ylab, colors, cex, lty, lwd, ylim, xlim, 
+          factor.names, ci.style, band.transparency, band.colors, type, ticks, 
+          alternating, rotx, roty, grid, layout,
+          rescale.axis, transform.x, ticks.x, show.strip.values, key.args, 
+          use.splines, residuals.color, residuals.pch, residuals.cex, smooth.residuals,
+          residuals.smooth.color, show.fitted, span)
+{ 
   closest <- function(x, x0) apply(outer(x, x0, FUN=function(x, x0) abs(x - x0)), 1, which.min)
   .mod <- function(a, b) ifelse( (d <- a %% b) == 0, b, d)
   .modc <- function(a) .mod(a, length(colors))
@@ -227,17 +241,22 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
     id.cex <- id$cex
     id.labels <- id$labels
   }
-  
   if (missing(lattice)) lattice <- NULL
   lattice <- applyDefaults(lattice, defaults=list(
-    layout=NULL, key.args=list(),
+    layout=NULL, #key.args=list(),  
     strip=list(factor.names=TRUE, values=!partial.residuals),
     array=list(row=1, col=1, nrow=1, ncol=1, more=FALSE),
     arg="lattice"
   ))
+  lattice$key.args <- applyDefaults(lattice$key.args, defaults=list(
+    space="top", border=FALSE, fontfamily="sans", cex=.75, cex.title=1, 
+    arg="key.args"
+  ))
+  if("x" %in% names(lattice$key.args)) lattice$key.args[["space"]] <- NULL
   if (missing(layout)) layout <- lattice$layout
   if (missing(key.args)){
-    lattice$key.args[["between.columns"]] <- if(is.null(lattice$key.args[["between.columns"]])) 0 else
+    lattice$key.args[["between.columns"]] <- 
+      if(is.null(lattice$key.args[["between.columns"]])) 0 else
       lattice$key.args[["between.columns"]]
     key.args <- lattice$key.args
   }
@@ -288,7 +307,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
   residuals <- if (partial.residuals) x$residuals else NULL
   if (!is.null(residuals) && !is.null(id.labels)) names(residuals) <- id.labels
   partial.residuals.range <- x$partial.residuals.range
-  
+ 
   if (!rescale.axis){
     x$lower[!is.na(x$lower)] <- trans.inverse(x$lower[!is.na(x$lower)])
     x$upper[!is.na(x$upper)] <- trans.inverse(x$upper[!is.na(x$upper)])
@@ -309,6 +328,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
   }
   has.se <- !is.null(x$se)
   n.predictors <- ncol(x) - 1 - 3*has.se
+
   if (n.predictors == 1){
     predictor <- names(x)[1]
     if (is.list(xlab)) xlab <- xlab[[predictor]]
@@ -319,16 +339,17 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
         range(c(x$lower, x$upper), na.rm=TRUE) else range(x$fit, na.rm=TRUE)
       ylim <- if (!any(is.na(ylim))) ylim else c(range[1] - .025*(range[2] - range[1]),
                                                  range[2] + .025*(range[2] - range[1]))
-      tickmarks <- if (type == "response" && rescale.axis) make.ticks(ylim,
-                                                                      link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
+      tickmarks <- if (type == "response" && rescale.axis) 
+        make.ticks(ylim, link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
       else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
       levs <- levels(x[,1])
       plot <- xyplot(eval(parse(
         text=paste("fit ~ as.numeric(", names(x)[1], ")"))),
         strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
-        panel=function(x, y, lower, upper, has.se, ...){
+        panel=function(x, y, lower, upper, has.se, ...){ 
           if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
           good <- !is.na(y)
+          if(!all(!good)){
           if (has.se){
             if (ci.style == "bars"){
               larrows(x0=x[good], y0=lower[good], x1=x[good], y1=upper[good], angle=90,
@@ -351,7 +372,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
             panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                        thresholds, threshold.labels, adj=c(1,0), cex=0.75)
           }
-        },
+        }},
         ylim=ylim,
         ylab=ylab,
         xlab=if (is.na(xlab)) names(x)[1] else xlab,
@@ -379,8 +400,8 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
                                max(partial.residuals.range[2], range[2] + .025*(range[2] - range[1])))
       else c(min(original.inverse(partial.residuals.range[1]), range[1] - .025*(range[2] - range[1])),
              max(original.inverse(partial.residuals.range[2]), range[2] + .025*(range[2] - range[1])))
-      tickmarks <- if (type == "response" && rescale.axis) make.ticks(ylim,
-                                                                      link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
+      tickmarks <- if (type == "response" && rescale.axis) 
+              make.ticks(ylim, link=trans.link, inverse=trans.inverse, at=ticks$at, n=ticks$n)
       else make.ticks(ylim, link=I, inverse=I, at=ticks$at, n=ticks$n)
       nm <- names(x)[1]
       x.vals <- x.data[, nm]
@@ -416,9 +437,10 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
       plot <- xyplot(eval(parse(
         text=paste("fit ~ trans(", x.var, ")"))),
         strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE)),
-        panel=function(x, y, x.vals, rug, lower, upper, has.se, ...){
+        panel=function(x, y, x.vals, rug, lower, upper, has.se, ...){ 
           if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
           good <- !is.na(y)
+          if(!all(!good)){
           axis.length <- diff(range(x))
           effect.llines(x[good], y[good], lwd=lwd, col=colors[1], ...)
           if (rug && is.null(residuals)) lrug(trans(x.vals))
@@ -464,7 +486,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
             }
           }
           
-        },
+        }},
         ylim=ylim,
         xlim=suppressWarnings(trans(xlm)),
         ylab=ylab,
@@ -523,11 +545,15 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
     if (is.factor(x[,x.var])){
       if (ci.style == "auto") ci.style <- "bars"
       levs <- levels(x[,x.var])
-      key <- list(title=predictors[z.var], cex.title=1, border=TRUE,
+      key <- list(title=predictors[z.var], #cex.title=1, border=TRUE,
                   text=list(as.character(zvals)),
-                  lines=list(col=colors[.modc(1:length(zvals))], lty=lines[.modl(1:length(zvals))], lwd=lwd),
-                  points=list(col=colors[.modc(1:length(zvals))], pch=symbols[.mods(1:length(zvals))]),
-                  columns = if ("x" %in% names(key.args)) 1 else find.legend.columns(length(zvals)))
+                  lines=list(col=colors[.modc(1:length(zvals))], 
+                             lty=lines[.modl(1:length(zvals))], lwd=lwd),
+                  points=list(col=colors[.modc(1:length(zvals))], 
+                              pch=symbols[.mods(1:length(zvals))]),
+                  columns = if ("x" %in% names(key.args)) 1 else 
+                    find.legend.columns(length(zvals), 
+                        space=if("x" %in% names(key.args)) "top" else key.args$space))
       for (k in names(key.args)) key[k] <- key.args[k]
       if (show.strip.values && n.predictors > 2){
         for (pred in predictors[-c(x.var, z.var)]){
@@ -539,11 +565,12 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
                    if (n.predictors > 2) paste(" |",
                                                paste(predictors[-c(x.var, z.var)], collapse="*"))))),
         strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE), sep=" = "),
-        panel=function(x, y, subscripts, z, lower, upper, show.se, ...){
+        panel=function(x, y, subscripts, z, lower, upper, show.se, ...){ 
           if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
           for (i in 1:length(zvals)){
             sub <- z[subscripts] == zvals[i]
             good <- !is.na(y[sub])
+            if(!all(!good)){
             os <- if(show.se)
               (i - (length(zvals) + 1)/2) * (2/(length(zvals)-1)) *
               .01 * (length(zvals) - 1) else 0
@@ -555,7 +582,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
                       angle=90, code=3, col=eval(colors[.modc(i)]),
                       length=.125*cex/1.5)
             }
-          }
+          }}
           if (has.thresholds){
             panel.abline(h=thresholds, lty=3)
             panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
@@ -609,10 +636,13 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
         trans <- I
         make.ticks(xlm, link=I, inverse=I, at=at, n=n)
       }
-      key <- list(title=predictors[z.var], cex.title=1, border=TRUE,
+      key <- list(title=predictors[z.var], #cex.title=1, border=TRUE,
                   text=list(as.character(zvals)),
-                  lines=list(col=colors[.modc(1:length(zvals))], lty=lines[.modl(1:length(zvals))], lwd=lwd),
-                  columns = if ("x" %in% names(key.args)) 1 else find.legend.columns(length(zvals)))
+                  lines=list(col=colors[.modc(1:length(zvals))], 
+                             lty=lines[.modl(1:length(zvals))], lwd=lwd),
+                  columns = if ("x" %in% names(key.args)) 1 else 
+                             find.legend.columns(length(zvals), 
+                             if("x" %in% names(key.args)) "top" else key.args$space))
       for (k in names(key.args)) key[k] <- key.args[k]
       if (show.strip.values && n.predictors > 2){
         for (pred in predictors[-c(x.var, z.var)]){
@@ -624,13 +654,14 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
                    if (n.predictors > 2) paste(" |",
                                                paste(predictors[-c(x.var, z.var)], collapse="*"))))),
         strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE), sep=" = "),
-        panel=function(x, y, subscripts, x.vals, rug, z, lower, upper, show.se, ...){
+        panel=function(x, y, subscripts, x.vals, rug, z, lower, upper, show.se, ...){ 
           if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
           if (rug && is.null(residuals)) lrug(trans(x.vals))
           axis.length <- diff(range(x))
           for (i in 1:length(zvals)){
             sub <- z[subscripts] == zvals[i]
             good <- !is.na(y[sub])
+            if(!all(!good)){
             effect.llines(x[sub][good], y[sub][good], lwd=lwd, type='l',
                           col=colors[.modc(i)], lty=lines[.modl(i)], cex=cex, ...)
             if(show.se){
@@ -649,7 +680,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
                             alpha=band.transparency, use.splines=use.splines)
               }
             }
-          }
+          }}
           if (has.thresholds){
             panel.abline(h=thresholds, lty=3)
             panel.text(rep(current.panel.limits()$xlim[1], length(thresholds)),
@@ -706,9 +737,11 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
       text=paste("fit ~ as.numeric(", predictors[x.var], ") |",
                  paste(predictors[-x.var], collapse="*")))),
       strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE), sep=" = "),
-      panel=function(x, y, subscripts, lower, upper, has.se, ...){
+      panel=function(x, y, subscripts, lower, upper, has.se, ...){ 
         if (grid) ticksGrid(x=1:length(levs), y=tickmarks$at)
         good <- !is.na(y)
+        no.points <- all(!good) # skip arrows and lines if no.points==TRUE
+        if(!no.points){
         if (has.se){
           if (ci.style == "bars"){
             larrows(x0=x[good], y0=lower[subscripts][good], x1=x[good], y1=upper[subscripts][good],
@@ -731,7 +764,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
           panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                      thresholds, threshold.labels, adj=c(1,0), cex=0.75)
         }
-      },
+      }},
       ylim=ylim,
       ylab=ylab,
       xlab=if (is.na(xlab)) predictors[x.var] else xlab,
@@ -791,9 +824,10 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
       text=paste("fit ~ trans(", predictors[x.var], ") |",
                  paste(predictors[-x.var], collapse="*")))),
       strip=function(...) strip.default(..., strip.names=c(factor.names, TRUE), sep=" = "),
-      panel=function(x, y, subscripts, x.vals, rug, lower, upper, has.se, ...){
+      panel=function(x, y, subscripts, x.vals, rug, lower, upper, has.se, ...){ 
         if (grid) ticksGrid(x=tickmarks.x$at, y=tickmarks$at)
         good <- !is.na(y)
+        if(!all(!good)){
         effect.llines(x[good], y[good], lwd=lwd, col=colors[1], ...)
         if (rug && is.null(residuals)) lrug(trans(x.vals))
         if (has.se){
@@ -847,7 +881,7 @@ plot.eff <- function(x, x.var, z.var=which.min(levels), main=paste(effect, "effe
           panel.text(rep(current.panel.limits()$xlim[2], length(thresholds)),
                      thresholds, threshold.labels, adj=c(1,0), cex=0.75)
         }
-      },
+      }},
       ylim=ylim,
       xlim=suppressWarnings(trans(xlm)),
       ylab=ylab,
@@ -876,14 +910,14 @@ plot.efflist <- function(x, selection, rows, cols, ask=FALSE, graphics=TRUE, lat
   lattice <- if(missing(lattice)) list() else lattice
   if (!missing(selection)){
     if (is.character(selection)) selection <- gsub(" ", "", selection)
-    return(plot(x[[selection]], ...))
+    return(plot(x[[selection]], lattice=lattice, ...))
   }
   effects <- gsub(":", "*", names(x))
   if (ask){
     repeat {
       selection <- menu(effects, graphics=graphics, title="Select Term to Plot")
       if (selection == 0) break
-      else print(plot(x[[selection]], ...))
+      else print(plot(x[[selection]], lattice=lattice, ...))
     }
   }
   else {
